// FIXME: some of these class types have items in them with arenas, but we can't hold a reference
//        to an arena, so check to make sure we copy the items out and don't store the
//        the item with the arena in it.

/* NSPR header files */
#undef HAVE_LONG_LONG           /* FIXME: both Python.h and nspr.h define HAVE_LONG_LONG  */
#include "nspr.h"
#include "cert.h"
#include "nss.h"
#include "ssl.h"
#include "sslt.h"
#include "key.h"
#include "pk11pub.h"

/* ========================================================================== */
/* =============================== PK11Slot Class =========================== */
/* ========================================================================== */

typedef struct {
    PyObject_HEAD
    PK11SlotInfo *slot;
} PK11Slot;

/* ========================================================================== */
/* ================================ CertDB Class ============================ */
/* ========================================================================== */

typedef struct {
    PyObject_HEAD
    CERTCertDBHandle *handle;
} CertDB;


/* ========================================================================== */
/* ============================ Certificate Class =========================== */
/* ========================================================================== */

typedef struct {
    PyObject_HEAD
    CERTCertificate *cert;
    PyObject *py_subject_public_key_info;
} Certificate;


/* ========================================================================== */
/* ============================= PrivateKey Class =========================== */
/* ========================================================================== */

typedef struct {
    PyObject_HEAD
    SECKEYPrivateKey *private_key;
} PrivateKey;

/* ========================================================================== */
/* =============================== SecItem Class ============================ */
/* ========================================================================== */

typedef enum SECItemKindEnum {
    SECITEM_unknown,
    SECITEM_dist_name,
    SECITEM_session_id,
    SECITEM_signed_data,
    SECITEM_signature,
    SECITEM_algorithm,
} SECItemKind;

typedef struct {
    PyObject_HEAD
    SECItem item;
    SECItemKind kind;    
} SecItem;

/* ========================================================================== */
/* ============================ RSAPublicKey Class ========================== */
/* ========================================================================== */

typedef struct {
    PyObject_HEAD
    PyObject *py_modulus;
    PyObject *py_exponent;
} RSAPublicKey;

/* ========================================================================== */
/* ============================ DSAPublicKey Class ========================== */
/* ========================================================================== */

typedef struct {
    PyObject_HEAD
    PyObject *py_pqg_params;
    PyObject *py_public_value;
} DSAPublicKey;

/* ========================================================================== */
/* ============================ KEYPQGParams Class ========================== */
/* ========================================================================== */

typedef struct {
    PyObject_HEAD
    PyObject *py_prime;
    PyObject *py_subprime;
    PyObject *py_base;
} KEYPQGParams;

/* ========================================================================== */
/* ======================== SignatureAlgorithm Class ======================== */
/* ========================================================================== */

typedef struct {
    PyObject_HEAD
    SECAlgorithmID id;
    PyObject *py_id;
    PyObject *py_parameters;
} SignatureAlgorithm;

/* ========================================================================== */
/* ============================= SignedData Class =========================== */
/* ========================================================================== */

typedef struct {
    PyObject_HEAD
    PRArenaPool *arena;
    CERTSignedData signed_data;
    PyObject *py_data;
    PyObject *py_algorithm;
    PyObject *py_signature;
} SignedData;

/* ========================================================================== */
/* ============================= PublicKey Class =========================== */
/* ========================================================================== */

typedef struct {
    PyObject_HEAD
    SECKEYPublicKey *pk;
    PyObject *py_pk11slot;
    PyObject *py_rsa_key;
    PyObject *py_dsa_key;
} PublicKey;

/* ========================================================================== */
/* ======================== SubjectPublicKeyInfo Class ====================== */
/* ========================================================================== */

typedef struct {
    PyObject_HEAD
    PRArenaPool *arena;
    CERTSubjectPublicKeyInfo spki;
    PyObject *py_algorithm;
    PyObject *py_public_key;
} SubjectPublicKeyInfo;

typedef struct {
    PyTypeObject *pk11slot_type;
    PyTypeObject *certdb_type;
    PyTypeObject *certificate_type;
    PyTypeObject *private_key_type;
    PyTypeObject *sec_item_type;
    PyObject *(*Certificate_new_from_cert)(CERTCertificate *cert);
    PyObject *(*PrivateKey_new_from_private_key)(SECKEYPrivateKey *private_key);
    PyObject *(*SecItem_new_from_sec_item)(SECItem *item, SECItemKind type);
    PyObject *(*cert_distnames_new_from_CERTDistNames)(CERTDistNames *names);
    CERTDistNames *(*cert_distnames_as_CERTDistNames)(PyObject *py_distnames);
} PyNSPR_NSS_C_API_Type;

#ifdef NSPR_NSS_MODULE

#define PyPK11Slot_Check(op) PyObject_TypeCheck(op, &PK11SlotType)
#define PyCertDB_Check(op) PyObject_TypeCheck(op, &CertDBType)
#define PyCertificate_Check(op) PyObject_TypeCheck(op, &CertificateType)
#define PyPrivateKey_Check(op) PyObject_TypeCheck(op, &PrivateKeyType)
#define PySecItem_Check(op) PyObject_TypeCheck(op, &SecItemType)

PyObject *
PK11Slot_new_from_slotinfo(PK11SlotInfo *slot);

#else  /* not NSPR_NSS_MODULE */

#define CertDBType (*nspr_nss_c_api.certdb_type)
#define CertificateType (*nspr_nss_c_api.certificate_type)
#define PrivateKeyType (*nspr_nss_c_api.private_key_type)
#define SecItemType (*nspr_nss_c_api.sec_item_type)

#define PyPK11Slot_Check(op) PyObject_TypeCheck(op, nspr_nss_c_api.pk11slot_type)
#define PyCertDB_Check(op) PyObject_TypeCheck(op, nspr_nss_c_api.certdb_type)
#define PyCertificate_Check(op) PyObject_TypeCheck(op, nspr_nss_c_api.certificate_type)
#define PyPrivateKey_Check(op) PyObject_TypeCheck(op, nspr_nss_c_api.private_key_type)
#define PySecItem_Check(op) PyObject_TypeCheck(op, nspr_nss_c_api.sec_item_type)

static PyNSPR_NSS_C_API_Type nspr_nss_c_api;

#define Certificate_new_from_cert (*nspr_nss_c_api.Certificate_new_from_cert)
#define PrivateKey_new_from_private_key (*nspr_nss_c_api.PrivateKey_new_from_private_key)
#define SecItem_new_from_sec_item (*nspr_nss_c_api.SecItem_new_from_sec_item)
#define cert_distnames_new_from_CERTDistNames (*nspr_nss_c_api.cert_distnames_new_from_CERTDistNames)
#define cert_distnames_as_CERTDistNames (*nspr_nss_c_api.cert_distnames_as_CERTDistNames)

static int
import_nspr_nss_c_api(void)
{
    PyObject *module = NULL;
    PyObject *c_api_object = NULL;
    void *api = NULL;

    if ((module = PyImport_ImportModule("nss.nss")) == NULL)
        return -1;

    if ((c_api_object = PyObject_GetAttrString(module, "_C_API")) == NULL) {
        Py_DECREF(module);
        return -1;
    }
    
    if (!(PyCObject_Check(c_api_object))) {
        Py_DECREF(c_api_object);
        Py_DECREF(module);
        return -1;
    }

    if ((api = PyCObject_AsVoidPtr(c_api_object)) == NULL) {
        Py_DECREF(c_api_object);
        Py_DECREF(module);
        return -1;
    }

    memcpy(&nspr_nss_c_api, api, sizeof(nspr_nss_c_api));
    Py_DECREF(c_api_object);
    Py_DECREF(module);
    return 0;
}

#endif /* NSPR_NSS_MODULE */
